import { useEffect, useState, createContext, useContext } from 'react';
import { supabase } from '../supabaseClient';

const AuthContext = createContext();

const AuthProvider = ({ children }) => {
  const [user, setUser] = useState();
  const [authLoading, setAuthLoading] = useState(true);

  useEffect(() => {
    const session = supabase.auth.session();
    setUser(session?.user ?? null);

    const { data: authListener } = supabase.auth.onAuthStateChange(
      (_event, session) => {
        setUser(session?.user ?? null);
      }
    );

    authListener && setAuthLoading(false);

    return () => {
      authListener?.unsubscribe();
      console.log('unloading auth listener');
    };
  }, []);

  const signUp = async (email, password, username) => {
    const { user, session, error } = await supabase.auth.signUp({
      email,
      password,
    });

    // TODO: also update the users table with the user nickname

    if (error) return { error };

    return { user, session };
  };

  const signIn = async (email, password) => {
    const { user, session, error } = await supabase.auth.signIn({
      email,
      password,
    });

    if (error) return { error };

    return { user, session };
  };

  const signOut = async () => {
    const { error } = await supabase.auth.signOut();

    if (error) return error.message;
    return { status: 'success' };
  };

  const updateUserMetaData = async (newMetaData) => {
    const { error } = await supabase.auth.update({
      data: { ...newMetaData },
    });

    if (error) return error.message;
    return { status: 'success' };
  };

  const value = {
    user,
    signUp,
    signIn,
    signOut,
    updateUserMetaData,
  };

  return (
    <AuthContext.Provider value={value}>
      {!authLoading && children}
    </AuthContext.Provider>
  );
};

const useAuth = () => {
  const context = useContext(AuthContext);

  if (!context) throw new Error('useAuth must be used within AuthProvider');

  return context;
};

export { AuthProvider, useAuth };
